@@ -1,19 +1,15 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use std::convert::TryFrom;
-use std::ops::{Deref, DerefMut};
-use std::rc::Rc;
-use std::slice;
+use std::{
+    ops::{Deref, DerefMut},
+    rc::Rc,
+    slice,
+};
 
-use crate::enums::{Format, SurfaceType};
-use crate::error::Error;
 #[cfg(feature = "use_glib")]
 use glib::translate::*;
 
-use crate::surface::Surface;
-use crate::utils::status_to_result;
-use crate::BorrowError;
-use std::fmt;
+use crate::{utils::status_to_result, BorrowError, Error, Format, Surface, SurfaceType};
 
 declare_surface!(ImageSurface, SurfaceType::Image);
 
@@ -181,12 +177,14 @@ unsafe impl Send for ImageSurfaceDataOwned {}
 unsafe impl Sync for ImageSurfaceDataOwned {}
 
 impl ImageSurfaceDataOwned {
+    #[inline]
     pub fn into_inner(self) -> ImageSurface {
         self.surface
     }
 }
 
 impl AsRef<[u8]> for ImageSurfaceDataOwned {
+    #[inline]
     fn as_ref(&self) -> &[u8] {
         let len = (self.surface.stride() as usize) * (self.surface.height() as usize);
         unsafe {
@@ -200,6 +198,7 @@ impl AsRef<[u8]> for ImageSurfaceDataOwned {
 }
 
 impl AsMut<[u8]> for ImageSurfaceDataOwned {
+    #[inline]
     fn as_mut(&mut self) -> &mut [u8] {
         let len = (self.surface.stride() as usize) * (self.surface.height() as usize);
         unsafe {
@@ -215,12 +214,14 @@ impl AsMut<[u8]> for ImageSurfaceDataOwned {
 impl Deref for ImageSurfaceDataOwned {
     type Target = [u8];
 
+    #[inline]
     fn deref(&self) -> &Self::Target {
         self.as_ref()
     }
 }
 
 impl DerefMut for ImageSurfaceDataOwned {
+    #[inline]
     fn deref_mut(&mut self) -> &mut Self::Target {
         self.as_mut()
     }
@@ -255,6 +256,7 @@ impl<'a> ImageSurfaceData<'a> {
 }
 
 impl<'a> Drop for ImageSurfaceData<'a> {
+    #[inline]
     fn drop(&mut self) {
         if self.dirty {
             self.surface.mark_dirty()
@@ -265,24 +267,20 @@ impl<'a> Drop for ImageSurfaceData<'a> {
 impl<'a> Deref for ImageSurfaceData<'a> {
     type Target = [u8];
 
+    #[inline]
     fn deref(&self) -> &[u8] {
         self.slice
     }
 }
 
 impl<'a> DerefMut for ImageSurfaceData<'a> {
+    #[inline]
     fn deref_mut(&mut self) -> &mut [u8] {
         self.dirty = true;
         self.slice
     }
 }
 
-impl<'a> fmt::Display for ImageSurfaceData<'a> {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "ImageSurfaceData")
-    }
-}
-
 // Workaround for cairo not having a direct way to check if the surface is finished.
 // See: https://gitlab.freedesktop.org/cairo/cairo/-/issues/406
 fn is_finished(surface: &ImageSurface) -> bool {
@@ -338,7 +336,8 @@ mod tests {
     #[cfg(feature = "use_glib")]
     #[test]
     fn surface_gvalues() {
-        use glib::ToValue;
+        use glib::prelude::*;
+
         let surface = ImageSurface::create(Format::ARgb32, 10, 10).unwrap();
         let value = surface.to_value();
         assert_eq!(value.get::<ImageSurface>().unwrap().width(), 10);
