@@ -1,16 +1,12 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use std::convert::TryFrom;
-use std::fmt;
-use std::ops::Deref;
+use std::{fmt, ops::Deref};
 
-use crate::enums::{Format, SurfaceType};
-use crate::error::Error;
-use crate::surface::Surface;
+use ffi::CGContextRef;
 #[cfg(feature = "use_glib")]
 use glib::translate::*;
 
-use ffi::CGContextRef;
+use crate::{Error, Format, Surface, SurfaceType};
 
 declare_surface!(QuartzSurface, SurfaceType::Quartz);
 
