@@ -1,76 +1,70 @@
-// Copyright 2013-2015, The Gtk-rs Project Developers.
-// See the COPYRIGHT file at the top-level directory of this distribution.
-// Licensed under the MIT license, see the LICENSE file or <http://opensource.org/licenses/MIT>
+// Take a look at the license at the top of the repository in the LICENSE file.
+
+#[cfg(feature = "use_glib")]
+use std::marker::PhantomData;
+use std::{ops::Deref, ptr};
 
-use enums::SurfaceType;
-use ffi;
 #[cfg(feature = "use_glib")]
 use glib::translate::*;
-use std::fmt;
 
-use surface::Surface;
+use crate::{Error, Surface, SurfaceType};
 
 #[derive(Debug)]
 pub struct XCBDrawable(pub u32);
 
 impl XCBDrawable {
+    #[inline]
     fn to_raw_none(&self) -> u32 {
         self.0
     }
 }
 
-impl fmt::Display for XCBDrawable {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "XCBDrawable")
-    }
-}
-
 #[derive(Debug)]
 pub struct XCBPixmap(pub u32);
 
 impl XCBPixmap {
+    #[inline]
     fn to_raw_none(&self) -> u32 {
         self.0
     }
 }
 
-impl fmt::Display for XCBPixmap {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "XCBPixmap")
-    }
-}
-
 #[derive(Debug)]
-pub struct XCBConnection(pub *mut ffi::xcb_connection_t);
+#[doc(alias = "xcb_connection_t")]
+pub struct XCBConnection(pub ptr::NonNull<ffi::xcb_connection_t>);
 
 impl XCBConnection {
+    #[inline]
     pub fn to_raw_none(&self) -> *mut ffi::xcb_connection_t {
-        self.0
+        self.0.as_ptr()
     }
 
+    #[inline]
     pub unsafe fn from_raw_none(ptr: *mut ffi::xcb_connection_t) -> XCBConnection {
-        assert!(!ptr.is_null());
-        XCBConnection(ptr)
+        debug_assert!(!ptr.is_null());
+        XCBConnection(ptr::NonNull::new_unchecked(ptr))
     }
 
-    pub unsafe fn from_raw_borrow(ptr: *mut ffi::xcb_connection_t) -> XCBConnection {
-        assert!(!ptr.is_null());
-        XCBConnection(ptr)
+    #[inline]
+    pub unsafe fn from_raw_borrow(ptr: *mut ffi::xcb_connection_t) -> Borrowed<XCBConnection> {
+        debug_assert!(!ptr.is_null());
+        Borrowed::new(XCBConnection(ptr::NonNull::new_unchecked(ptr)))
     }
 
+    #[inline]
     pub unsafe fn from_raw_full(ptr: *mut ffi::xcb_connection_t) -> XCBConnection {
-        assert!(!ptr.is_null());
-        XCBConnection(ptr)
+        debug_assert!(!ptr.is_null());
+        XCBConnection(ptr::NonNull::new_unchecked(ptr))
     }
 }
 
 #[cfg(feature = "use_glib")]
 impl<'a> ToGlibPtr<'a, *mut ffi::xcb_connection_t> for &'a XCBConnection {
-    type Storage = &'a XCBConnection;
+    type Storage = PhantomData<&'a XCBConnection>;
 
     #[inline]
     fn to_glib_none(&self) -> Stash<'a, *mut ffi::xcb_connection_t, &'a XCBConnection> {
-        Stash(self.to_raw_none(), *self)
+        Stash(self.to_raw_none(), PhantomData)
     }
 }
 
@@ -85,7 +79,7 @@ impl FromGlibPtrNone<*mut ffi::xcb_connection_t> for XCBConnection {
 #[cfg(feature = "use_glib")]
 impl FromGlibPtrBorrow<*mut ffi::xcb_connection_t> for XCBConnection {
     #[inline]
-    unsafe fn from_glib_borrow(ptr: *mut ffi::xcb_connection_t) -> XCBConnection {
+    unsafe fn from_glib_borrow(ptr: *mut ffi::xcb_connection_t) -> Borrowed<XCBConnection> {
         Self::from_raw_borrow(ptr)
     }
 }
@@ -99,52 +93,52 @@ impl FromGlibPtrFull<*mut ffi::xcb_connection_t> for XCBConnection {
 }
 
 impl Clone for XCBConnection {
+    #[inline]
     fn clone(&self) -> XCBConnection {
         unsafe { Self::from_raw_none(self.to_raw_none()) }
     }
 }
 
-impl fmt::Display for XCBConnection {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "XCBConnection")
-    }
-}
-
 #[derive(Debug)]
-pub struct XCBRenderPictFormInfo(pub *mut ffi::xcb_render_pictforminfo_t);
+#[doc(alias = "xcb_render_pictforminfo_t")]
+pub struct XCBRenderPictFormInfo(pub ptr::NonNull<ffi::xcb_render_pictforminfo_t>);
 
 impl XCBRenderPictFormInfo {
+    #[inline]
     pub fn to_raw_none(&self) -> *mut ffi::xcb_render_pictforminfo_t {
-        self.0
+        self.0.as_ptr()
     }
 
+    #[inline]
     pub unsafe fn from_raw_none(ptr: *mut ffi::xcb_render_pictforminfo_t) -> XCBRenderPictFormInfo {
-        assert!(!ptr.is_null());
-        XCBRenderPictFormInfo(ptr)
+        debug_assert!(!ptr.is_null());
+        XCBRenderPictFormInfo(ptr::NonNull::new_unchecked(ptr))
     }
 
+    #[inline]
     pub unsafe fn from_raw_borrow(
         ptr: *mut ffi::xcb_render_pictforminfo_t,
-    ) -> XCBRenderPictFormInfo {
-        assert!(!ptr.is_null());
-        XCBRenderPictFormInfo(ptr)
+    ) -> Borrowed<XCBRenderPictFormInfo> {
+        debug_assert!(!ptr.is_null());
+        Borrowed::new(XCBRenderPictFormInfo(ptr::NonNull::new_unchecked(ptr)))
     }
 
+    #[inline]
     pub unsafe fn from_raw_full(ptr: *mut ffi::xcb_render_pictforminfo_t) -> XCBRenderPictFormInfo {
-        assert!(!ptr.is_null());
-        XCBRenderPictFormInfo(ptr)
+        debug_assert!(!ptr.is_null());
+        XCBRenderPictFormInfo(ptr::NonNull::new_unchecked(ptr))
     }
 }
 
 #[cfg(feature = "use_glib")]
 impl<'a> ToGlibPtr<'a, *mut ffi::xcb_render_pictforminfo_t> for &'a XCBRenderPictFormInfo {
-    type Storage = &'a XCBRenderPictFormInfo;
+    type Storage = PhantomData<&'a XCBRenderPictFormInfo>;
 
     #[inline]
     fn to_glib_none(
         &self,
     ) -> Stash<'a, *mut ffi::xcb_render_pictforminfo_t, &'a XCBRenderPictFormInfo> {
-        Stash(self.to_raw_none(), *self)
+        Stash(self.to_raw_none(), PhantomData)
     }
 }
 
@@ -159,7 +153,9 @@ impl FromGlibPtrNone<*mut ffi::xcb_render_pictforminfo_t> for XCBRenderPictFormI
 #[cfg(feature = "use_glib")]
 impl FromGlibPtrBorrow<*mut ffi::xcb_render_pictforminfo_t> for XCBRenderPictFormInfo {
     #[inline]
-    unsafe fn from_glib_borrow(ptr: *mut ffi::xcb_render_pictforminfo_t) -> XCBRenderPictFormInfo {
+    unsafe fn from_glib_borrow(
+        ptr: *mut ffi::xcb_render_pictforminfo_t,
+    ) -> Borrowed<XCBRenderPictFormInfo> {
         Self::from_raw_borrow(ptr)
     }
 }
@@ -173,48 +169,48 @@ impl FromGlibPtrFull<*mut ffi::xcb_render_pictforminfo_t> for XCBRenderPictFormI
 }
 
 impl Clone for XCBRenderPictFormInfo {
+    #[inline]
     fn clone(&self) -> XCBRenderPictFormInfo {
         unsafe { Self::from_raw_none(self.to_raw_none()) }
     }
 }
 
-impl fmt::Display for XCBRenderPictFormInfo {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "XCBRenderPictFormInfo")
-    }
-}
-
 #[derive(Debug)]
-pub struct XCBScreen(pub *mut ffi::xcb_screen_t);
+#[doc(alias = "xcb_screen_t")]
+pub struct XCBScreen(pub ptr::NonNull<ffi::xcb_screen_t>);
 
 impl XCBScreen {
+    #[inline]
     pub fn to_raw_none(&self) -> *mut ffi::xcb_screen_t {
-        self.0
+        self.0.as_ptr()
     }
 
+    #[inline]
     pub unsafe fn from_raw_none(ptr: *mut ffi::xcb_screen_t) -> XCBScreen {
-        assert!(!ptr.is_null());
-        XCBScreen(ptr)
+        debug_assert!(!ptr.is_null());
+        XCBScreen(ptr::NonNull::new_unchecked(ptr))
     }
 
-    pub unsafe fn from_raw_borrow(ptr: *mut ffi::xcb_screen_t) -> XCBScreen {
-        assert!(!ptr.is_null());
-        XCBScreen(ptr)
+    #[inline]
+    pub unsafe fn from_raw_borrow(ptr: *mut ffi::xcb_screen_t) -> Borrowed<XCBScreen> {
+        debug_assert!(!ptr.is_null());
+        Borrowed::new(XCBScreen(ptr::NonNull::new_unchecked(ptr)))
     }
 
+    #[inline]
     pub unsafe fn from_raw_full(ptr: *mut ffi::xcb_screen_t) -> XCBScreen {
-        assert!(!ptr.is_null());
-        XCBScreen(ptr)
+        debug_assert!(!ptr.is_null());
+        XCBScreen(ptr::NonNull::new_unchecked(ptr))
     }
 }
 
 #[cfg(feature = "use_glib")]
 impl<'a> ToGlibPtr<'a, *mut ffi::xcb_screen_t> for &'a XCBScreen {
-    type Storage = &'a XCBScreen;
+    type Storage = PhantomData<&'a XCBScreen>;
 
     #[inline]
     fn to_glib_none(&self) -> Stash<'a, *mut ffi::xcb_screen_t, &'a XCBScreen> {
-        Stash(self.to_raw_none(), *self)
+        Stash(self.to_raw_none(), PhantomData)
     }
 }
 
@@ -229,7 +225,7 @@ impl FromGlibPtrNone<*mut ffi::xcb_screen_t> for XCBScreen {
 #[cfg(feature = "use_glib")]
 impl FromGlibPtrBorrow<*mut ffi::xcb_screen_t> for XCBScreen {
     #[inline]
-    unsafe fn from_glib_borrow(ptr: *mut ffi::xcb_screen_t) -> XCBScreen {
+    unsafe fn from_glib_borrow(ptr: *mut ffi::xcb_screen_t) -> Borrowed<XCBScreen> {
         Self::from_raw_borrow(ptr)
     }
 }
@@ -243,67 +239,44 @@ impl FromGlibPtrFull<*mut ffi::xcb_screen_t> for XCBScreen {
 }
 
 impl Clone for XCBScreen {
+    #[inline]
     fn clone(&self) -> XCBScreen {
         unsafe { Self::from_raw_none(self.to_raw_none()) }
     }
 }
 
-impl fmt::Display for XCBScreen {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "XCBScreen")
-    }
-}
-
-pub struct XCBSurface(Surface);
-
-impl std::ops::Deref for XCBSurface {
-    type Target = Surface;
-
-    fn deref(&self) -> &Surface {
-        &self.0
-    }
-}
-
-impl std::convert::TryFrom<Surface> for XCBSurface {
-    type Error = Surface;
-
-    fn try_from(surface: Surface) -> Result<Self, Surface> {
-        if surface.get_type() == SurfaceType::Xcb {
-            Ok(Self(surface))
-        } else {
-            Err(surface)
-        }
-    }
-}
+declare_surface!(XCBSurface, SurfaceType::Xcb);
 
 impl XCBSurface {
+    #[doc(alias = "cairo_xcb_surface_create")]
     pub fn create(
         connection: &XCBConnection,
         drawable: &XCBDrawable,
         visual: &XCBVisualType,
         width: i32,
         height: i32,
-    ) -> Self {
+    ) -> Result<Self, Error> {
         unsafe {
-            Self(Surface::from_raw_full(ffi::cairo_xcb_surface_create(
+            Self::from_raw_full(ffi::cairo_xcb_surface_create(
                 connection.to_raw_none(),
                 drawable.to_raw_none(),
                 visual.to_raw_none(),
                 width,
                 height,
-            )))
+            ))
         }
     }
 
+    #[doc(alias = "cairo_xcb_surface_create_for_bitmap")]
     pub fn create_for_bitmap(
         connection: &XCBConnection,
         screen: &XCBScreen,
         bitmap: &XCBPixmap,
         width: i32,
         height: i32,
-    ) -> Self {
+    ) -> Result<Self, Error> {
         unsafe {
-            Self(Surface::from_raw_full(
+            Ok(Self(Surface::from_raw_full(
                 ffi::cairo_xcb_surface_create_for_bitmap(
                     connection.to_raw_none(),
                     screen.to_raw_none(),
@@ -311,10 +284,11 @@ impl XCBSurface {
                     width,
                     height,
                 ),
-            ))
+            )?))
         }
     }
 
+    #[doc(alias = "cairo_xcb_surface_create_with_xrender_format")]
     pub fn create_with_xrender_format(
         connection: &XCBConnection,
         screen: &XCBScreen,
@@ -322,9 +296,9 @@ impl XCBSurface {
         format: &XCBRenderPictFormInfo,
         width: i32,
         height: i32,
-    ) -> Self {
+    ) -> Result<Self, Error> {
         unsafe {
-            Self(Surface::from_raw_full(
+            Ok(Self(Surface::from_raw_full(
                 ffi::cairo_xcb_surface_create_with_xrender_format(
                     connection.to_raw_none(),
                     screen.to_raw_none(),
@@ -333,15 +307,23 @@ impl XCBSurface {
                     width,
                     height,
                 ),
-            ))
+            )?))
         }
     }
 
-    pub fn set_size(&self, width: i32, height: i32) {
+    #[doc(alias = "cairo_xcb_surface_set_size")]
+    pub fn set_size(&self, width: i32, height: i32) -> Result<(), Error> {
         unsafe { ffi::cairo_xcb_surface_set_size(self.to_raw_none(), width, height) }
+        self.status()
     }
 
-    pub fn set_drawable(&self, drawable: &XCBDrawable, width: i32, height: i32) {
+    #[doc(alias = "cairo_xcb_surface_set_drawable")]
+    pub fn set_drawable(
+        &self,
+        drawable: &XCBDrawable,
+        width: i32,
+        height: i32,
+    ) -> Result<(), Error> {
         unsafe {
             ffi::cairo_xcb_surface_set_drawable(
                 self.to_raw_none(),
@@ -350,40 +332,46 @@ impl XCBSurface {
                 height,
             )
         }
+        self.status()
     }
 }
 
 #[derive(Debug)]
-pub struct XCBVisualType(pub *mut ffi::xcb_visualtype_t);
+#[doc(alias = "xcb_visualtype_t")]
+pub struct XCBVisualType(pub ptr::NonNull<ffi::xcb_visualtype_t>);
 
 impl XCBVisualType {
+    #[inline]
     pub fn to_raw_none(&self) -> *mut ffi::xcb_visualtype_t {
-        self.0
+        self.0.as_ptr()
     }
 
+    #[inline]
     pub unsafe fn from_raw_none(ptr: *mut ffi::xcb_visualtype_t) -> XCBVisualType {
-        assert!(!ptr.is_null());
-        XCBVisualType(ptr)
+        debug_assert!(!ptr.is_null());
+        XCBVisualType(ptr::NonNull::new_unchecked(ptr))
     }
 
-    pub unsafe fn from_raw_borrow(ptr: *mut ffi::xcb_visualtype_t) -> XCBVisualType {
-        assert!(!ptr.is_null());
-        XCBVisualType(ptr)
+    #[inline]
+    pub unsafe fn from_raw_borrow(ptr: *mut ffi::xcb_visualtype_t) -> Borrowed<XCBVisualType> {
+        debug_assert!(!ptr.is_null());
+        Borrowed::new(XCBVisualType(ptr::NonNull::new_unchecked(ptr)))
     }
 
+    #[inline]
     pub unsafe fn from_raw_full(ptr: *mut ffi::xcb_visualtype_t) -> XCBVisualType {
-        assert!(!ptr.is_null());
-        XCBVisualType(ptr)
+        debug_assert!(!ptr.is_null());
+        XCBVisualType(ptr::NonNull::new_unchecked(ptr))
     }
 }
 
 #[cfg(feature = "use_glib")]
 impl<'a> ToGlibPtr<'a, *mut ffi::xcb_visualtype_t> for &'a XCBVisualType {
-    type Storage = &'a XCBVisualType;
+    type Storage = PhantomData<&'a XCBVisualType>;
 
     #[inline]
     fn to_glib_none(&self) -> Stash<'a, *mut ffi::xcb_visualtype_t, &'a XCBVisualType> {
-        Stash(self.to_raw_none(), *self)
+        Stash(self.to_raw_none(), PhantomData)
     }
 }
 
@@ -398,7 +386,7 @@ impl FromGlibPtrNone<*mut ffi::xcb_visualtype_t> for XCBVisualType {
 #[cfg(feature = "use_glib")]
 impl FromGlibPtrBorrow<*mut ffi::xcb_visualtype_t> for XCBVisualType {
     #[inline]
-    unsafe fn from_glib_borrow(ptr: *mut ffi::xcb_visualtype_t) -> XCBVisualType {
+    unsafe fn from_glib_borrow(ptr: *mut ffi::xcb_visualtype_t) -> Borrowed<XCBVisualType> {
         Self::from_raw_borrow(ptr)
     }
 }
@@ -412,24 +400,22 @@ impl FromGlibPtrFull<*mut ffi::xcb_visualtype_t> for XCBVisualType {
 }
 
 impl Clone for XCBVisualType {
+    #[inline]
     fn clone(&self) -> XCBVisualType {
         unsafe { Self::from_raw_none(self.to_raw_none()) }
     }
 }
 
-impl fmt::Display for XCBVisualType {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "XCBVisualType")
-    }
-}
-
-impl ::device::Device {
-    pub fn get_connection(&self) -> XCBConnection {
+impl crate::device::Device {
+    #[doc(alias = "cairo_xcb_device_get_connection")]
+    #[doc(alias = "get_connection")]
+    pub fn connection(&self) -> XCBConnection {
         unsafe {
             XCBConnection::from_raw_full(ffi::cairo_xcb_device_get_connection(self.to_raw_none()))
         }
     }
 
+    #[doc(alias = "cairo_xcb_device_debug_cap_xshm_version")]
     pub fn debug_cap_xshm_version(&self, major_version: i32, minor_version: i32) {
         unsafe {
             ffi::cairo_xcb_device_debug_cap_xshm_version(
